import requests


class Runs():
    def apply_terraform_run(self, run_id, message="This action was performed via the rest API."):
        url = self.url + 'runs/{}/actions/apply'.format(run_id)
        payload = {
            "comment": message
        }
        return self._post_handler(url=url, json=payload)

    def force_execute_run(self, run_id):
        url = self.url + 'runs/{}/actions/force-execute'.format(run_id)
        response = requests.post(url=url, headers=self.headers)
        self._error_handler(response)
        return response

    def run_terraform_workspace(self, workspace_id, message, destroy=False):
        url = self.url + 'runs'
        payload = {
            "data": {
                "attributes": {
                    "is-destroy": destroy,
                    "message": message
                },
                "type": "runs",
                "relationships": {
                    "workspace": {
                        "data": {
                            "type": "workspaces",
                            "id": workspace_id
                        }
                    }
                }
            }
        }

        return self._post_handler(url=url, json=payload)

    def get_workspace_runs(self, workspace_id):
        url = self.url + 'workspaces/{}/runs'.format(workspace_id)
        return self._tfe_api_get(url)

    def discard_run(self, run_id, comment):
        url = self.url + '/runs/{}/actions/discard'.format(run_id)
        payload = {
            "comment": comment
        }
        return self._post_handler(url=url, json=payload)

    def cancel_run(self, run_id, comment, force_cancel=False):
        cancel = 'cancel'
        if force_cancel:
            cancel = 'force-cancel'
        url = self.url + 'runs/{}/actions/{}'.format(run_id, cancel)
        payload = {
            "comment": comment
        }
        return self._post_handler(url=url, json=payload)



class Teams():
    def show_user(self, user_id):
        url = self.url + 'users/{}'.format(user_id)
        return self._get_handler(url)

    def get_teams(self, organization_name):
        url = self.url + 'organizations/{}/teams'.format(organization_name)
        return self._get_handler(url)

    def create_team(self, organization, name, is_managed_workspace=True):
        url = self.url + 'organizations/{}/teams'.format(organization)
        payload = {
            "data": {
                "type": "teams",
                "attributes": {
                    "name": name,
                    "organization-access": {
                        "manage-workspaces": is_managed_workspace
                    }
                }
            }
        }
        return self._post_handler(url, json=payload)

    def show_team_information(self, team_id):
        """
        Returns information on a given team id.
        :param team_id: String of team ID.
        """
        url = self.url + 'teams/{}'.format(team_id)
        return self._get_handler(url)

    def delete_team(self, team_id):
        """
        Removes a given team from Terraform Enterprise
        :param team_id: String of team ID.
        """
        url = self.url + 'teams/{}'.format(team_id)
        return self._delete_handler(url)



import requests
import logging as log
import json
from .pyterprise_exceptions import UnauthorizedError, InternalServerError
from .organizations import Organziations
from .plans import Plans
from .teams import Teams
from .runs import Runs
from .variables import Variables
from .workspaces import Workspaces
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

"""
Copyright (c) 2018 The Python Packaging Authority

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


class Client(Organziations, Plans, Teams, Runs, Variables, Workspaces):
    """
    Client class which inherits subclasses for different method types as defined in the TFE API Documentation. Set token
    and V2 API URL using the non default 'init' construcutor method.
    """
    log.basicConfig(
        level=log.WARNING
    )

    def __init__(self):
        return

    def init(self, token, url, ssl_verification=True, version='v2'):
        self.token = token
        self.url = url + '/api/{}/'.format(version)
        self.headers = {
            'Content-Type': 'application/vnd.api+json',
            'Authorization': 'Bearer {}'.format(token)
        }
        
        self.ssl_verification = ssl_verification

    # HTTP helper methods
    def _get_handler(self, url):
        response = requests.get(url=url, verify=self.ssl_verification, headers=self.headers)
        self._error_handler(response)
        return response.content

    def _post_handler(self, url, json):
        response = requests.post(url=url, verify=self.ssl_verification, headers=self.headers, json=json)
        self._error_handler(response)
        return response.content

    def _patch_handler(self, url, json):
        response = requests.patch(url=url, verify=self.ssl_verification, headers=self.headers, json=json)
        self._error_handler(response)
        return response.content

    def _delete_handler(self, url):
        response = requests.delete(url, verify=self.ssl_verification)
        self._error_handler(response)
        return response.content

    @staticmethod
    def _error_handler(response):
            if response.status_code in range(400, 499):
                raise UnauthorizedError(
                    message=response.content,
                    errors=response.status_code
                )
            if response.status_code in range(500, 504):
                raise InternalServerError(
                    message=response.content,
                    errors=response.status_code
                )

            if response.status_code not in range(200, 202):
                log.warning('{}: {}'.format(response.url, response.status_code))

